@@ -1,5 +1,5 @@
 /* makepath.c -- Ensure that a directory path exists.
-   Copyright (C) 1990, 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1990, 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -50,6 +50,10 @@ char *alloca ();
 # define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
 #endif
 
+#ifndef S_IRWXUGO
+# define S_IRWXUGO (S_IRWXU | S_IRWXG | S_IRWXO)
+#endif
+
 #if STDC_HEADERS
 # include <stdlib.h>
 #endif
@@ -71,24 +75,48 @@ extern int errno;
 # endif
 #endif
 
+#ifndef S_ISUID
+# define S_ISUID 04000
+#endif
+#ifndef S_ISGID
+# define S_ISGID 02000
+#endif
+#ifndef S_ISVTX
+# define S_ISVTX 01000
+#endif
+#ifndef S_IRUSR
+# define S_IRUSR 0200
+#endif
 #ifndef S_IWUSR
 # define S_IWUSR 0200
 #endif
-
 #ifndef S_IXUSR
 # define S_IXUSR 0100
 #endif
 
+#ifndef S_IRWXU
+# define S_IRWXU (S_IRUSR | S_IWUSR | S_IXUSR)
+#endif
+
 #define WX_USR (S_IWUSR | S_IXUSR)
 
-#ifdef __MSDOS__
-typedef int uid_t;
-typedef int gid_t;
+/* Include this before libintl.h so we get our definition of PARAMS. */
+#include "makepath.h"
+
+#if HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
+#if ENABLE_NLS
+# include <libintl.h>
+# define _(Text) gettext (Text)
+#else
+# define _(Text) Text
 #endif
 
 #include "save-cwd.h"
-#include "makepath.h"
 #include "error.h"
+#include "quote.h"
 
 void strip_trailing_slashes ();
 
@@ -99,9 +127,9 @@ void strip_trailing_slashes ();
 	 Restore working directory.  */			\
       if (do_chdir)					\
 	{						\
-	  int fail = restore_cwd (&cwd, NULL, NULL);	\
+	  int _fail = restore_cwd (&cwd, NULL, NULL);	\
 	  free_cwd (&cwd);				\
-	  if (fail)					\
+	  if (_fail)					\
 	    return 1;					\
 	}						\
     }							\
@@ -115,6 +143,58 @@ void strip_trailing_slashes ();
     }							\
   while (0)
 
+/* Attempt to create directory DIR (aka DIRPATH) with the specified MODE.
+   If CREATED_DIR_P is non-NULL, set *CREATED_DIR_P to non-zero if this
+   function creates DIR and to zero otherwise.  Give a diagnostic and
+   return non-zero if DIR cannot be created or cannot be determined to
+   exist already.  Use DIRPATH in any diagnostic, not DIR.
+   Note that if DIR already exists, this function returns zero
+   (indicating success) and sets *CREATED_DIR_P to zero.  */
+
+int
+make_dir (const char *dir, const char *dirpath, mode_t mode, int *created_dir_p)
+{
+  int fail = 0;
+  int created_dir;
+
+  created_dir = (mkdir (dir, mode) == 0);
+
+  if (!created_dir)
+    {
+      struct stat stats;
+      int saved_errno = errno;
+
+      /* The mkdir and stat calls below may appear to be reversed.
+	 They are not.  It is important to call mkdir first and then to
+	 call stat (to distinguish the three cases) only if mkdir fails.
+	 The alternative to this approach is to `stat' each directory,
+	 then to call mkdir if it doesn't exist.  But if some other process
+	 were to create the directory between the stat & mkdir, the mkdir
+	 would fail with EEXIST.  */
+
+      if (stat (dir, &stats))
+	{
+	  error (0, saved_errno, _("cannot create directory %s"),
+		 quote (dirpath));
+	  fail = 1;
+	}
+      else if (!S_ISDIR (stats.st_mode))
+	{
+	  error (0, 0, _("%s exists but is not a directory"), quote (dirpath));
+	  fail = 1;
+	}
+      else
+	{
+	  /* DIR (aka DIRPATH) already exists and is a directory. */
+	}
+    }
+
+  if (created_dir_p)
+    *created_dir_p = created_dir;
+
+  return fail;
+}
+
 /* Ensure that the directory ARGPATH exists.
    Remove any trailing slashes from ARGPATH before calling this function.
 
@@ -162,7 +242,7 @@ make_path (const char *argpath,
       char *dirpath;
 
       /* Temporarily relax umask in case it's overly restrictive.  */
-      int oldmask = umask (0);
+      mode_t oldmask = umask (0);
 
       /* Make a copy of ARGPATH that we can scribble NULs on.  */
       dirpath = (char *) alloca (strlen (argpath) + 1);
@@ -174,9 +254,9 @@ make_path (const char *argpath,
 	 their owners, we need to fix their permissions after making them.  */
       if (((parent_mode & WX_USR) != WX_USR)
 	  || ((owner != (uid_t) -1 || group != (gid_t) -1)
-	      && (parent_mode & 07000) != 0))
+	      && (parent_mode & (S_ISUID | S_ISGID | S_ISVTX)) != 0))
 	{
-	  tmp_mode = 0700;
+	  tmp_mode = S_IRWXU;
 	  re_protect = 1;
 	}
       else
@@ -203,7 +283,8 @@ make_path (const char *argpath,
 
       while (1)
 	{
-	  int newly_created_dir = 1;
+	  int newly_created_dir;
+	  int fail;
 
 	  /* slash points to the leftmost unprocessed component of dirpath.  */
 	  basename_dir = slash;
@@ -217,40 +298,18 @@ make_path (const char *argpath,
 	  if (!do_chdir)
 	    basename_dir = dirpath;
 
-	  /* The mkdir and stat calls below appear to be reversed.
-	     They are not.  It is important to call mkdir first and then to
-	     call stat (to distinguish the three cases) only if mkdir fails.
-	     The alternative to this approach is to `stat' each directory,
-	     then to call mkdir if it doesn't exist.  But if some other process
-	     were to create the directory between the stat & mkdir, the mkdir
-	     would fail with EEXIST.  */
-
 	  *slash = '\0';
-	  if (mkdir (basename_dir, tmp_mode))
+	  fail = make_dir (basename_dir, dirpath, tmp_mode, &newly_created_dir);
+	  if (fail)
 	    {
-	      if (stat (basename_dir, &stats))
-		{
-		  error (0, errno, "cannot create directory `%s'", dirpath);
-		  CLEANUP;
-		  return 1;
-		}
-	      else if (!S_ISDIR (stats.st_mode))
-		{
-		  error (0, 0, "`%s' exists but is not a directory", dirpath);
-		  CLEANUP;
-		  return 1;
-		}
-	      else
-		{
-		  /* DIRPATH already exists and is a directory. */
-		  newly_created_dir = 0;
-		}
+	      CLEANUP;
+	      return 1;
 	    }
 
 	  if (newly_created_dir)
 	    {
 	      if (verbose_fmt_string)
-		fprintf (stderr, verbose_fmt_string, dirpath);
+		error (0, 0, verbose_fmt_string, quote (dirpath));
 
 	      if ((owner != (uid_t) -1 || group != (gid_t) -1)
 		  && chown (basename_dir, owner, group)
@@ -259,7 +318,8 @@ make_path (const char *argpath,
 #endif
 		  )
 		{
-		  error (0, errno, "%s", dirpath);
+		  error (0, errno, _("cannot change owner and/or group of %s"),
+			 quote (dirpath));
 		  CLEANUP;
 		  return 1;
 		}
@@ -280,7 +340,8 @@ make_path (const char *argpath,
 	     stat and mkdir process O(n^2) file name components.  */
 	  if (do_chdir && chdir (basename_dir) < 0)
 	    {
-	      error (0, errno, "cannot chdir to directory, %s", dirpath);
+	      error (0, errno, _("cannot chdir to directory, %s"),
+		     quote (dirpath));
 	      CLEANUP;
 	      return 1;
 	    }
@@ -299,12 +360,8 @@ make_path (const char *argpath,
       /* We're done making leading directories.
 	 Create the final component of the path.  */
 
-      /* The path could end in "/." or contain "/..", so test
-	 if we really have to create the directory.  */
-
-      if (stat (basename_dir, &stats) && mkdir (basename_dir, mode))
+      if (make_dir (basename_dir, dirpath, mode, NULL))
 	{
-	  error (0, errno, "cannot create directory `%s'", dirpath);
 	  CLEANUP;
 	  return 1;
 	}
@@ -313,9 +370,9 @@ make_path (const char *argpath,
       umask (oldmask);
 
       if (verbose_fmt_string != NULL)
-	error (0, 0, verbose_fmt_string, dirpath);
+	error (0, 0, verbose_fmt_string, quote (dirpath));
 
-      if (owner != (uid_t) -1 && group != (gid_t) -1)
+      if (owner != (uid_t) -1 || group != (gid_t) -1)
 	{
 	  if (chown (basename_dir, owner, group)
 #ifdef AFS
@@ -323,17 +380,24 @@ make_path (const char *argpath,
 #endif
 	      )
 	    {
-	      error (0, errno, "cannot chown %s", dirpath);
-	      retval = 1;
-	    }
-	  /* chown may have turned off some permission bits we wanted.  */
-	  if ((mode & 07000) != 0 && chmod (basename_dir, mode))
-	    {
-	      error (0, errno, "cannot chmod %s", dirpath);
+	      error (0, errno, _("cannot change owner and/or group of %s"),
+		     quote (dirpath));
 	      retval = 1;
 	    }
 	}
 
+      /* The above chown may have turned off some permission bits in MODE.
+	 Another reason we may have to use chmod here is that mkdir(2) is
+	 required to honor only the file permission bits.  In particular,
+	 it need not honor the `special' bits, so if MODE includes any
+	 special bits, set them here.  */
+      if ((mode & ~S_IRWXUGO)
+	  && chmod (basename_dir, mode))
+	{
+	  error (0, errno, _("cannot change permissions of %s"), quote (dirpath));
+	  retval = 1;
+	}
+
       CLEANUP_CWD;
 
       /* If the mode for leading directories didn't include owner "wx"
@@ -344,7 +408,8 @@ make_path (const char *argpath,
 	  *(p->dirname_end) = '\0';
 	  if (chmod (dirpath, parent_mode))
 	    {
-	      error (0, errno, "%s", dirpath);
+	      error (0, errno, "cannot change permissions of %s",
+		     quote (dirpath));
 	      retval = 1;
 	    }
 	}
@@ -357,7 +422,7 @@ make_path (const char *argpath,
 
       if (!S_ISDIR (stats.st_mode))
 	{
-	  error (0, 0, "`%s' exists but is not a directory", dirpath);
+	  error (0, 0, _("%s exists but is not a directory"), quote (dirpath));
 	  return 1;
 	}
 
@@ -376,12 +441,14 @@ make_path (const char *argpath,
 #endif
 	      )
 	    {
-	      error (0, errno, "%s", dirpath);
+	      error (0, errno, _("cannot change owner and/or group of %s"),
+		     quote (dirpath));
 	      retval = 1;
 	    }
 	  if (chmod (dirpath, mode))
 	    {
-	      error (0, errno, "%s", dirpath);
+	      error (0, errno, _("cannot change permissions of %s"),
+				 quote (dirpath));
 	      retval = 1;
 	    }
 	}
