@@ -1,4 +1,4 @@
-/* Copyright (C) 1991-1999, 2000 Free Software Foundation, Inc.
+/* Copyright (C) 1991-1999, 2000, 2001 Free Software Foundation, Inc.
 
    NOTE: The canonical source of this file is maintained with the GNU C Library.
    Bugs can be reported to bug-glibc@prep.ai.mit.edu.
@@ -289,7 +289,7 @@ static const CHAR_T zeroes[16] = /* "0000000000000000" */
 	 else if (to_uppcase)						      \
 	   memcpy_uppcase (p, (s), _n);					      \
 	 else								      \
-	   MEMCPY ((PTR) p, (PTR) (s), _n))
+	   MEMCPY ((PTR) p, (const PTR) (s), _n))
 
 #ifdef COMPILE_WIDE
 # define widen(os, ws, l) \
@@ -457,8 +457,7 @@ static CHAR_T const month_name[][10] =
     return _strftime_copytm (s, maxsize, format, &tmcopy ut_argument);
   }
 # undef my_strftime
-# define my_strftime(S, Maxsize, Format, Tp) \
-  _strftime_copytm (S, Maxsize, Format, Tp)
+# define my_strftime _strftime_copytm
 #endif
 
 
@@ -516,6 +515,9 @@ my_strftime (s, maxsize, format, tp ut_argument)
   size_t i = 0;
   CHAR_T *p = s;
   const CHAR_T *f;
+#if DO_MULTIBYTE && !defined COMPILE_WIDE
+  const char *format_end = NULL;
+#endif
 
   zone = NULL;
 #if HAVE_TM_ZONE
@@ -608,10 +610,15 @@ my_strftime (s, maxsize, format, tp ut_argument)
 	  {
 	    mbstate_t mbstate = mbstate_zero;
 	    size_t len = 0;
+	    size_t fsize;
+
+	    if (! format_end)
+	      format_end = f + strlen (f) + 1;
+	    fsize = format_end - f;
 
 	    do
 	      {
-		size_t bytes = mbrlen (f + len, (size_t) -1, &mbstate);
+		size_t bytes = mbrlen (f + len, fsize - len, &mbstate);
 
 		if (bytes == 0)
 		  break;
@@ -786,10 +793,11 @@ my_strftime (s, maxsize, format, tp ut_argument)
 	    goto bad_format;
 #ifdef _NL_CURRENT
 	  if (! (modifier == 'E'
-		 && (*(subfmt = (CHAR_T *) _NL_CURRENT (LC_TIME,
-							NLW(ERA_D_T_FMT)))
+		 && (*(subfmt =
+		       (const CHAR_T *) _NL_CURRENT (LC_TIME,
+						     NLW(ERA_D_T_FMT)))
 		     != '\0')))
-	    subfmt = (CHAR_T *) _NL_CURRENT (LC_TIME, NLW(D_T_FMT));
+	    subfmt = (const CHAR_T *) _NL_CURRENT (LC_TIME, NLW(D_T_FMT));
 #else
 # if HAVE_STRFTIME
 	  goto underlying_strftime;
@@ -801,8 +809,10 @@ my_strftime (s, maxsize, format, tp ut_argument)
 	subformat:
 	  {
 	    CHAR_T *old_start = p;
-	    size_t len = my_strftime (NULL, (size_t) -1, subfmt, tp);
-	    add (len, my_strftime (p, maxsize - i, subfmt, tp));
+	    size_t len = my_strftime (NULL, (size_t) -1, subfmt,
+				      tp ut_argument);
+	    add (len, my_strftime (p, maxsize - i, subfmt,
+				   tp ut_argument));
 
 	    if (to_uppcase)
 	      while (old_start < p)
@@ -878,10 +888,10 @@ my_strftime (s, maxsize, format, tp ut_argument)
 	    goto bad_format;
 #ifdef _NL_CURRENT
 	  if (! (modifier == L_('E')
-		 && (*(subfmt = (CHAR_T *)_NL_CURRENT (LC_TIME,
-						       NLW(ERA_D_FMT)))
+		 && (*(subfmt =
+		       (const CHAR_T *)_NL_CURRENT (LC_TIME, NLW(ERA_D_FMT)))
 		     != L_('\0'))))
-	    subfmt = (CHAR_T *) _NL_CURRENT (LC_TIME, NLW(D_FMT));
+	    subfmt = (const CHAR_T *) _NL_CURRENT (LC_TIME, NLW(D_FMT));
 	  goto subformat;
 #else
 # if HAVE_STRFTIME
@@ -1058,14 +1068,15 @@ my_strftime (s, maxsize, format, tp ut_argument)
 	  goto underlying_strftime;
 #endif
 
-	case L_('R'):		/* GNU extension.  */
+	case L_('R'):		/* ISO C99 extension.  */
 	  subfmt = L_("%H:%M");
 	  goto subformat;
 
 	case L_('r'):		/* POSIX.2 extension.  */
 #ifdef _NL_CURRENT
-	  if (*(subfmt = (CHAR_T *) _NL_CURRENT (LC_TIME,
-						 NLW(T_FMT_AMPM))) == L_('\0'))
+	  if (*(subfmt = (const CHAR_T *) _NL_CURRENT (LC_TIME,
+						       NLW(T_FMT_AMPM)))
+	      == L_('\0'))
 #endif
 	    subfmt = L_("%I:%M:%S %p");
 	  goto subformat;
@@ -1120,10 +1131,10 @@ my_strftime (s, maxsize, format, tp ut_argument)
 	    goto bad_format;
 #ifdef _NL_CURRENT
 	  if (! (modifier == L_('E')
-		 && (*(subfmt = (CHAR_T *) _NL_CURRENT (LC_TIME,
-							NLW(ERA_T_FMT)))
+		 && (*(subfmt =
+		       (const CHAR_T *) _NL_CURRENT (LC_TIME, NLW(ERA_T_FMT)))
 		     != L_('\0'))))
-	    subfmt = (CHAR_T *) _NL_CURRENT (LC_TIME, NLW(T_FMT));
+	    subfmt = (const CHAR_T *) _NL_CURRENT (LC_TIME, NLW(T_FMT));
 	  goto subformat;
 #else
 # if HAVE_STRFTIME
@@ -1150,8 +1161,8 @@ my_strftime (s, maxsize, format, tp ut_argument)
 	  DO_NUMBER (2, (tp->tm_yday - tp->tm_wday + 7) / 7);
 
 	case L_('V'):
-	case L_('g'):		/* GNU extension.  */
-	case L_('G'):		/* GNU extension.  */
+	case L_('g'):		/* ISO C99 extension.  */
+	case L_('G'):		/* ISO C99 extension.  */
 	  if (modifier == L_('E'))
 	    goto bad_format;
 	  {
@@ -1275,7 +1286,7 @@ my_strftime (s, maxsize, format, tp ut_argument)
 #endif
 	  break;
 
-	case L_('z'):		/* GNU extension.  */
+	case L_('z'):		/* ISO C99 extension.  */
 	  if (tp->tm_isdst < 0)
 	    break;
 
