@@ -95,6 +95,31 @@ static struct option const long_options[] =
   {0, 0, 0, 0}
 };
 
+static int
+mode_changed (const char *file, mode_t old_mode)
+{
+  struct stat new_stats;
+
+  if (lstat (file, &new_stats))
+    {
+      if (force_silent == 0)
+	error (0, errno, _("getting new attributes of %s"), quote (file));
+      return 0;
+    }
+
+#ifdef S_ISLNK
+  if (S_ISLNK (new_stats.st_mode)
+      && stat (file, &new_stats))
+    {
+      if (force_silent == 0)
+	error (0, errno, _("getting new attributes of %s"), quote (file));
+      return 0;
+    }
+#endif
+
+  return old_mode != new_stats.st_mode;
+}
+
 /* Tell the user how/if the MODE of FILE has been changed.
    CHANGED describes what (if anything) has happened. */
 
@@ -137,6 +162,8 @@ change_file_mode (const char *file, const struct mode_change *changes,
   struct stat file_stats;
   mode_t newmode;
   int errors = 0;
+  int fail;
+  int saved_errno;
 
   if (lstat (file, &file_stats))
     {
@@ -161,24 +188,21 @@ change_file_mode (const char *file, const struct mode_change *changes,
 
   newmode = mode_adjust (file_stats.st_mode, changes);
 
-  if (newmode != (file_stats.st_mode & CHMOD_MODE_BITS))
+  fail = chmod (file, newmode);
+  saved_errno = errno;
+
+  if (verbosity == V_high
+      || (verbosity == V_changes_only
+	  && !fail && mode_changed (file, file_stats.st_mode)))
+    describe_change (file, newmode, (fail ? CH_FAILED : CH_SUCCEEDED));
+
+  if (fail)
     {
-      int fail = chmod (file, newmode);
-      int saved_errno = errno;
-
-      if (verbosity == V_high || (verbosity == V_changes_only && !fail))
-	describe_change (file, newmode, (fail ? CH_FAILED : CH_SUCCEEDED));
-
-      if (fail)
-	{
-	  if (force_silent == 0)
-	    error (0, saved_errno, _("changing permissions of %s"),
-		   quote (file));
-	  errors = 1;
-	}
+      if (force_silent == 0)
+	error (0, saved_errno, _("changing permissions of %s"),
+	       quote (file));
+      errors = 1;
     }
-  else if (verbosity == V_high)
-    describe_change (file, newmode, CH_NO_CHANGE_REQUESTED);
 
   if (recurse && S_ISDIR (file_stats.st_mode))
     errors |= change_dir_mode (file, changes, &file_stats);
@@ -313,7 +337,12 @@ main (int argc, char **argv)
 	case '-':
 	case '=':
 	  if (modeind != 0 && modeind != thisind)
-	    error (1, 0, _("invalid mode"));
+	    {
+	      static char char_string[2] = {0, 0};
+	      char_string[0] = c;
+	      error (1, 0, _("invalid character %s in mode string %s"),
+		     quote_n (0, char_string), quote_n (1, argv[thisind]));
+	    }
 	  modeind = thisind;
 	  break;
 	case REFERENCE_FILE_OPTION:
@@ -351,7 +380,7 @@ main (int argc, char **argv)
 	     : mode_compile (argv[modeind], MODE_MASK_ALL));
 
   if (changes == MODE_INVALID)
-    error (1, 0, _("invalid mode"));
+    error (1, 0, _("invalid mode string: %s"), quote (argv[modeind]));
   else if (changes == MODE_MEMORY_EXHAUSTED)
     xalloc_die ();
   else if (changes == MODE_BAD_REFERENCE)
