@@ -1,5 +1,5 @@
 /* copy.c -- core functions for copying files and directories
-   Copyright (C) 89, 90, 91, 1995-2000 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2001 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,6 +67,11 @@ static int copy_internal PARAMS ((const char *src_path, const char *dst_path,
 				  int *copy_into_self,
 				  int *rename_succeeded));
 
+/* Pointers to the file names:  they're used in the diagnostic that is issued
+   when we detect the user is trying to copy a directory into itself.  */
+static char const *top_level_src_path;
+static char const *top_level_dst_path;
+
 /* The invocation name of this program.  */
 extern char *program_name;
 
@@ -392,8 +397,8 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
 {
   const struct stat *src_sb_link;
   const struct stat *dst_sb_link;
-  const struct stat *src_sb_no_link;
-  const struct stat *dst_sb_no_link;
+  struct stat tmp_dst_sb;
+  struct stat tmp_src_sb;
 
   int same_link;
   int same = (SAME_INODE (*src_sb, *dst_sb));
@@ -417,17 +422,15 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
 
       /* If both the source and destination files are symlinks (and we'll
 	 know this here IFF preserving symlinks (aka xstat == lstat),
-	 then it's ok.  */
+	 then it's ok -- as long as they are distinct.  */
       if (S_ISLNK (src_sb->st_mode) && S_ISLNK (dst_sb->st_mode))
-	return 1;
+	return ! same_name (src_path, dst_path);
 
       src_sb_link = src_sb;
       dst_sb_link = dst_sb;
     }
   else
     {
-      static struct stat tmp_dst_sb;
-      static struct stat tmp_src_sb;
       if (!same)
 	return 1;
 
@@ -491,7 +494,7 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
     return 1;
 
   /* If neither is a symlink, then it's ok as long as they aren't
-     links to the same file.  */
+     hard links to the same file.  */
   if (!S_ISLNK (src_sb_link->st_mode) && !S_ISLNK (dst_sb_link->st_mode))
     {
       if (!SAME_INODE (*src_sb_link, *dst_sb_link))
@@ -506,19 +509,25 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
     }
 
   /* It's ok to remove a destination symlink.  But that works only when we
-     unlink before opening the destination and when they're on the same
-     partition.  */
+     unlink before opening the destination and when the source and destination
+     files are on the same partition.  */
   if (x->unlink_dest_before_opening
       && S_ISLNK (dst_sb_link->st_mode))
-    return src_sb_link->st_dev == src_sb_link->st_dev;
+    return dst_sb_link->st_dev == src_sb_link->st_dev;
 
   if (x->xstat == lstat)
     {
-      static struct stat tmp_dst_sb;
-      static struct stat tmp_src_sb;
-      if (stat (dst_path, &tmp_dst_sb)
-	  || stat (src_path, &tmp_src_sb)
-	  || ! SAME_INODE (tmp_dst_sb, tmp_dst_sb))
+      if ( ! S_ISLNK (src_sb_link->st_mode))
+	tmp_src_sb = *src_sb_link;
+      else if (stat (src_path, &tmp_src_sb))
+	return 1;
+
+      if ( ! S_ISLNK (dst_sb_link->st_mode))
+	tmp_dst_sb = *dst_sb_link;
+      else if (stat (dst_path, &tmp_dst_sb))
+	return 1;
+
+      if ( ! SAME_INODE (tmp_src_sb, tmp_dst_sb))
 	return 1;
 
       /* FIXME: shouldn't this be testing whether we're making symlinks?  */
@@ -527,13 +536,6 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
 	  *return_now = 1;
 	  return 1;
 	}
-      src_sb_no_link = &tmp_src_sb;
-      dst_sb_no_link = &tmp_dst_sb;
-    }
-  else
-    {
-      src_sb_no_link = src_sb;
-      dst_sb_no_link = dst_sb;
     }
 
   return 0;
@@ -586,6 +588,10 @@ copy_internal (const char *src_path, const char *dst_path,
      find created files so as to not copy infinitely if a directory is
      copied into itself.  */
 
+  /* Associate the destination path with the source device and inode
+     so that if we encounter a matching dev/ino pair in the source tree
+     we can arrange to create a hard link between the corresponding names
+     in the destination tree.  */
   earlier_file = remember_copied (dst_path, src_sb.st_ino, src_sb.st_dev);
 
   src_mode = src_sb.st_mode;
@@ -626,6 +632,14 @@ copy_internal (const char *src_path, const char *dst_path,
 	      return 1;
 	    }
 
+	  if (S_ISDIR (src_type) && !S_ISDIR (dst_sb.st_mode))
+	    {
+	      error (0, 0,
+		     _("cannot overwrite non-directory %s with directory %s"),
+		     quote_n (0, dst_path), quote_n (1, src_path));
+	      return 1;
+	    }
+
 	  if (!S_ISDIR (src_type))
 	    {
 	      if (S_ISDIR (dst_sb.st_mode))
@@ -759,8 +773,21 @@ copy_internal (const char *src_path, const char *dst_path,
 	 directories).  */
       if (S_ISDIR (src_type))
 	{
-	  error (0, 0, _("won't create hard link %s to directory %s"),
-		 quote_n (0, dst_path), quote_n (1, earlier_file));
+	  /* If src_path and earlier_file refer to the same directory entry,
+	     then warn about copying a directory into itself.  */
+	  if (same_name (src_path, earlier_file))
+	    {
+	      error (0, 0, _("cannot copy a directory, %s, into itself, %s"),
+		     quote_n (0, top_level_src_path),
+		     quote_n (1, top_level_dst_path));
+	      *copy_into_self = 1;
+	    }
+	  else
+	    {
+	      error (0, 0, _("will not create hard link %s to directory %s"),
+		     quote_n (0, dst_path), quote_n (1, earlier_file));
+	    }
+
 	  goto un_backup;
 	}
 
@@ -806,7 +833,13 @@ copy_internal (const char *src_path, const char *dst_path,
 	  /* FIXME: this is a little fragile in that it relies on rename(2)
 	     failing with a specific errno value.  Expect problems on
 	     non-POSIX systems.  */
+	  error (0, 0, _("cannot move %s to a subdirectory of itself, %s"),
+		 quote_n (0, top_level_src_path),
+		 quote_n (1, top_level_dst_path));
 	  *copy_into_self = 1;
+	  /* FIXME-cleanup: Don't return zero here; adjust mv.c accordingly.
+	     The only caller that uses this code (mv.c) ends up setting its
+	     exit status to nonzero when copy_into_self is nonzero.  */
 	  return 0;
 	}
 
@@ -1170,6 +1203,17 @@ copy (const char *src_path, const char *dst_path,
   int move_mode = options->move_mode;
 
   assert (valid_options (options));
+
+  /* Record the file names: they're used in case of error, when copying
+     a directory into itself.  I don't like to make these tools do *any*
+     extra work in the common case when that work is solely to handle
+     exceptional cases, but in this case, I don't see a way to derive the
+     top level source and destination directory names where they're used.
+     An alternative is to use COPY_INTO_SELF and print the diagnostic
+     from every caller -- but I don't wan't to do that.  */
+  top_level_src_path = src_path;
+  top_level_dst_path = dst_path;
+
   return copy_internal (src_path, dst_path, nonexistent_dst, 0, NULL,
 			options, move_mode, copy_into_self, rename_succeeded);
 }
