@@ -1,5 +1,5 @@
 /* cp.c  -- file copying (main routines)
-   Copyright (C) 89, 90, 91, 1995-2000 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2001 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -124,7 +124,7 @@ static struct option const long_opts[] =
   {"preserve", no_argument, NULL, 'p'},
   {"recursive", no_argument, NULL, 'R'},
   {"remove-destination", no_argument, NULL, UNLINK_DEST_BEFORE_OPENING},
-  {"strip-trailing-slash", no_argument, NULL, STRIP_TRAILING_SLASHES_OPTION},
+  {"strip-trailing-slashes", no_argument, NULL, STRIP_TRAILING_SLASHES_OPTION},
   {"suffix", required_argument, NULL, 'S'},
   {"symbolic-link", no_argument, NULL, 's'},
   {"target-directory", required_argument, NULL, TARGET_DIRECTORY_OPTION},
@@ -172,6 +172,8 @@ Copy SOURCE to DEST, or multiple SOURCE(s) to DIRECTORY.\n\
                                  special files like FIFOs or /dev/zero\n\
       --remove-destination     remove each existing destination file before\n\
                                  attempting to open it (contrast with --force)\n\
+"));
+      printf (_("\
       --sparse=WHEN            control creation of sparse files\n\
   -R, --recursive              copy directories recursively\n\
       --strip-trailing-slashes remove any trailing slashes from each SOURCE\n\
@@ -330,6 +332,8 @@ re_protect (const char *const_dst_path, int src_offset,
    Return 0 if parent of CONST_DIRPATH exists as a directory with the proper
    permissions when done, otherwise 1. */
 
+/* FIXME: find a way to synch this function with the one in lib/makepath.c. */
+
 static int
 make_path_private (const char *const_dirpath, int src_offset, int mode,
 		   const char *verbose_fmt_string, struct dir_attr **attr_list,
@@ -516,13 +520,26 @@ do_copy (int n_files, char **file, const char *target_directory,
 	  char *arg_in_concat = NULL;
 	  char *arg = file[i];
 
+	  /* Trailing slashes are meaningful (i.e., maybe worth preserving)
+	     only in the source file names.  */
 	  if (remove_trailing_slashes)
 	    strip_trailing_slashes (arg);
 
 	  if (flag_path)
 	    {
-	      /* Append all of `arg' to `dest'.  */
-	      dst_path = path_concat (dest, arg, &arg_in_concat);
+	      char *arg_no_trailing_slash;
+
+	      /* Use `arg' without trailing slashes in constructing destination
+		 file names.  Otherwise, we can end up trying to create a
+		 directory via `mkdir ("dst/foo/"...', which is not portable.
+		 It fails, due to the trailing slash, on at least
+		 NetBSD 1.[34] systems.  */
+	      ASSIGN_STRDUPA (arg_no_trailing_slash, arg);
+	      strip_trailing_slashes (arg_no_trailing_slash);
+
+	      /* Append all of `arg' (minus any trailing slash) to `dest'.  */
+	      dst_path = path_concat (dest, arg_no_trailing_slash,
+				      &arg_in_concat);
 	      if (dst_path == NULL)
 		xalloc_die ();
 
