@@ -1,20 +1,20 @@
 /* dircolors - output commands to set the LS_COLOR environment variable
-   Copyright (C) 1994, 1995, 1997, 1998 H. Peter Anvin
-   Copyright (C) 96, 97, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1994, 1995, 1997, 1998, 1999, 2000 H. Peter Anvin
+   Copyright (C) 1996-2000 Free Software Foundation, Inc.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #if HAVE_CONFIG_H
 # include <config.h>
@@ -26,19 +26,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <stdio.h>
 
 #include "system.h"
-#include "getline.h"
-#include "long-options.h"
-#include "closeout.h"
-#include "error.h"
-#include "obstack.h"
 #include "dircolors.h"
+#include "error.h"
+#include "getline.h"
+#include "obstack.h"
+#include "quote.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "dircolors"
+
+#define AUTHORS "H. Peter Anvin"
 
 #define obstack_chunk_alloc malloc
 #define obstack_chunk_free free
 
-char *base_name ();
-char *strndup();
-
 enum Shell_syntax
 {
   SHELL_SYNTAX_BOURNE,
@@ -67,14 +68,14 @@ static const char *const slack_codes[] =
 {
   "NORMAL", "NORM", "FILE", "DIR", "LNK", "LINK",
   "SYMLINK", "ORPHAN", "MISSING", "FIFO", "PIPE", "SOCK", "BLK", "BLOCK",
-  "CHR", "CHAR", "EXEC", "LEFT", "LEFTCODE", "RIGHT", "RIGHTCODE", "END",
-  "ENDCODE", NULL
+  "CHR", "CHAR", "DOOR", "EXEC", "LEFT", "LEFTCODE", "RIGHT", "RIGHTCODE",
+  "END", "ENDCODE", NULL
 };
 
 static const char *const ls_codes[] =
 {
   "no", "no", "fi", "di", "ln", "ln", "ln", "or", "mi", "pi", "pi",
-  "so", "bd", "bd", "cd", "cd", "ex", "lc", "lc", "rc", "rc", "ec", "ec"
+  "so", "bd", "bd", "cd", "cd", "do", "ex", "lc", "lc", "rc", "rc", "ec", "ec"
 };
 
 static struct option const long_options[] =
@@ -83,15 +84,15 @@ static struct option const long_options[] =
     {"sh", no_argument, NULL, 'b'},
     {"csh", no_argument, NULL, 'c'},
     {"c-shell", no_argument, NULL, 'c'},
-    {"help", no_argument, NULL, 'h'},
     {"print-database", no_argument, NULL, 'p'},
-    {"version", no_argument, NULL, 'v'},
+    {GETOPT_HELP_OPTION_DECL},
+    {GETOPT_VERSION_OPTION_DECL},
     {NULL, 0, NULL, 0}
   };
 
 char *program_name;
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -115,7 +116,6 @@ file types and extensions.  Otherwise, a precompiled database is used.\n\
 For details on the format of these files, run `dircolors --print-database'.\n\
 "));
       puts (_("\nReport bugs to <bug-fileutils@gnu.org>."));
-      close_stdout ();
     }
 
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
@@ -126,7 +126,7 @@ xstrndup (const char *s, size_t n)
 {
   char *new = strndup (s, n);
   if (new == NULL)
-    error (EXIT_FAILURE, 0, _("virtual memory exhausted"));
+    xalloc_die ();
   return new;
 }
 
@@ -366,8 +366,9 @@ dc_parse_stream (FILE *fp, const char *filename)
 
       if (unrecognized && (state == ST_TERMSURE || state == ST_TERMYES))
 	{
-	  error (0, 0, _("%s:%lu: unrecognized keyword `%s'"),
-		 filename, (long unsigned) line_number, keywd);
+	  error (0, 0, _("%s:%lu: unrecognized keyword %s"),
+		 (filename ? quote (filename) : _("<internal>")),
+		 (long unsigned) line_number, keywd);
 	  err = 1;
 	}
 
@@ -399,7 +400,7 @@ dc_parse_file (const char *filename)
       fp = fopen (filename, "r");
       if (fp == NULL)
 	{
-	  error (0, errno, "%s", filename);
+	  error (0, errno, "%s", quote (filename));
 	  return 1;
 	}
     }
@@ -408,7 +409,7 @@ dc_parse_file (const char *filename)
 
   if (fp != stdin && fclose (fp) == EOF)
     {
-      error (0, errno, "%s", filename);
+      error (0, errno, "%s", quote (filename));
       return 1;
     }
 
@@ -428,7 +429,7 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "dircolors", GNU_PACKAGE, VERSION, usage);
+  atexit (close_stdout);
 
   while ((optc = getopt_long (argc, argv, "bcp", long_options, NULL)) != -1)
     switch (optc)
@@ -445,6 +446,10 @@ main (int argc, char **argv)
 	print_database = 1;
 	break;
 
+      case_GETOPT_HELP_CHAR;
+
+      case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
       default:
 	usage (1);
       }
@@ -527,7 +532,6 @@ dircolors' internal database"));
 	}
     }
 
-  close_stdout ();
 
   if (have_read_stdin && fclose (stdin) == EOF)
     error (EXIT_FAILURE, errno, _("standard input"));
