@@ -1,5 +1,5 @@
 /* remove.c -- core functions for removing files and directories
-   Copyright (C) 88, 90, 91, 1994-2000 Free Software Foundation, Inc.
+   Copyright (C) 88, 90, 91, 1994-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -171,7 +171,7 @@ hash_compare_active_dir_ents (void const *x, void const *y)
 {
   struct active_dir_ent const *a = x;
   struct active_dir_ent const *b = y;
-  return SAME_INODE (*a, *b);
+  return SAME_INODE (*a, *b) ? true : false;
 }
 
 /* A hash function for null-terminated char* strings using
