@@ -1,5 +1,5 @@
 /* touch -- change modification and access times of files
-   Copyright (C) 87, 1989-1991, 1995-2000 Free Software Foundation, Inc.
+   Copyright (C) 87, 1989-1991, 1995-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -217,7 +217,7 @@ usage (int status)
   else
     {
       printf (_("Usage: %s [OPTION]... FILE...\n"), program_name);
-      printf (_("  or : %s [-acm] MMDDhhmm[YY] FILE... (obsolescent)\n"),
+      printf (_("  or:  %s [-acm] MMDDhhmm[YY] FILE... (obsolescent)\n"),
 	      program_name);
       printf (_("\
 Update the access and modification times of each FILE to the current time.\n\
@@ -334,7 +334,9 @@ main (int argc, char **argv)
       date_set++;
     }
 
-  if (!date_set && optind < argc && !STREQ (argv[optind - 1], "--"))
+  /* The obsolescent `MMDDhhmm[YY]' form is valid IFF there are
+     two or more non-option arguments.  */
+  if (!date_set && 2 <= argc - optind && !STREQ (argv[optind - 1], "--"))
     {
       newtime = posixtime (argv[optind], PDS_TRAILING_YEAR);
       if (newtime != (time_t) -1)
